/*
 ============================================================================

  This file is part of the bbLean source code
  Copyright  2001-2003 The Blackbox for Windows Development Team
  Copyright  2004 grischka

  http://bb4win.sourceforge.net/bblean
  http://sourceforge.net/projects/bb4win

 ============================================================================

  bbLean and bb4win are free software, released under the GNU General
  Public License (GPL version 2 or later), with an extension that allows
  linking of proprietary modules under a controlled interface. This means
  that plugins etc. are allowed to be released under any license the author
  wishes. For details see:

  http://www.fsf.org/licenses/gpl.html
  http://www.fsf.org/licenses/gpl-faq.html#LinkingOverControlledInterface

  This program is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
  for more details.

 ============================================================================
*/

// bbWorkspaceWheel.cpp - switch desktop with mouse wheel

#include "../BBApi.h"

const char szVersion     [] = "bbWorkspaceWheel 0.1.0";
const char szAppName     [] = "bbWorkspaceWheel";
const char szInfoVersion [] = "0.1.0";
const char szInfoAuthor  [] = "grischka";
const char szInfoRelDate [] = "2005-02-05";
const char szInfoLink    [] = "http://bb4win.sourceforge.net/bblean";
const char szInfoEmail   [] = "grischka@users.sourceforge.net";

// ---------------------------------------------
#pragma data_seg( ".shared" )

static HHOOK hDeskHook      = 0;
static HWND DTWnd           = NULL;
static HWND BBhwnd          = NULL;
static bool is_win2k        = false;

#pragma data_seg()
#pragma comment(linker, "/SECTION:.shared,RWS")
// ---------------------------------------------

#define WM_MOUSEWHEEL 0x020A

typedef struct tagMOUSEHOOKSTRUCTEX : public tagMOUSEHOOKSTRUCT
{
    DWORD mouseData;
} MOUSEHOOKSTRUCTEX;

extern "C" BOOL WINAPI _DllMainCRTStartup(HINSTANCE hinstDLL, DWORD fdwReason, LPVOID lpvReserved)
{
    return TRUE;
}

//===========================================================================

LRESULT CALLBACK MouseProc(int nCode, WPARAM wParam, LPARAM lParam)
{
    if (nCode == HC_ACTION && wParam == WM_MOUSEWHEEL)
    {
        MOUSEHOOKSTRUCTEX* pmhExStruct = (MOUSEHOOKSTRUCTEX*)lParam;
        HWND hw = WindowFromPoint(pmhExStruct->pt);
        if (DTWnd == hw)
        {
            int delta = is_win2k
                ?(short)HIWORD(pmhExStruct->mouseData)
                :(short)LOWORD(pmhExStruct->dwExtraInfo)
                ;

            if (delta < 0)
                PostMessage(BBhwnd, BB_WORKSPACE, 1, 0);
            else
                PostMessage(BBhwnd, BB_WORKSPACE, 0, 0);

            return TRUE;
        }
    }
    return CallNextHookEx(hDeskHook, nCode, wParam, lParam);
}

//===========================================================================

int beginPlugin(HINSTANCE hPluginInstance)
{
    if (BBhwnd)
    {
        MessageBox(BBhwnd, "Dont load me twice!", szAppName, MB_OK|MB_SETFOREGROUND);
        return 1;
    }

    HWND hw;

    hw = FindWindow("BlackBoxClass", "BlackBox");
    if (NULL == hw)
    hw = FindWindow("xoblite", NULL);
    if (NULL == hw) return 1;
    BBhwnd = hw;

    hw = FindWindow("DesktopBackgroundClass", NULL);
    if (NULL == hw) hw = FindWindow("BBLeanDesktop", NULL);
    if (NULL == hw) hw = GetDesktopWindow();
    if (NULL == hw) return 1;
    DTWnd = hw;

    OSVERSIONINFO osInfo;
    ZeroMemory(&osInfo, sizeof osInfo);
    osInfo.dwOSVersionInfoSize = sizeof osInfo;
    GetVersionEx(&osInfo);

    bool usingNT = osInfo.dwPlatformId == VER_PLATFORM_WIN32_NT;
    bool usingWin2kXP = usingNT && osInfo.dwMajorVersion >= 5;
    is_win2k = usingWin2kXP;

    hDeskHook =  SetWindowsHookEx(WH_MOUSE, MouseProc, hPluginInstance, 0);
    if (NULL == hDeskHook) return 1;

    return 0;
}

void endPlugin(HINSTANCE hPluginInstance)
{
    UnhookWindowsHookEx(hDeskHook);
}

LPCSTR pluginInfo(int field)
{
    switch (field)
    {
        default:
        case 0: return szVersion;
        case 1: return szAppName;
        case 2: return szInfoVersion;
        case 3: return szInfoAuthor;
        case 4: return szInfoRelDate;
        case 5: return szInfoLink;
        case 6: return szInfoEmail;
    }
}

//===========================================================================
